#include "define.h"
#include "extern.h"

#ifndef NO_CURSES
#include <curses.h>
#include <signal.h>
#include <string.h>
#include <time.h>

#define CTRL(x) ((x) & 037)
#define PF (void)printw

void quit ( sig )
int sig;
{
	move ( LINES -1 , 0 );
	clrtoeol();
	curses_refresh();
	endwin();

	if ( sig > 0 ) {
		fprintf ( stderr, "Trapped signal %d\n", sig );
		exit ( sig );
	}
	exit ( sig );
}

void init_curses()
{
	int i;

	initscr();
	erase ();
	clear();
	curses_refresh();
	cbreak();

	if ( COLS < 80 ) {
		fprintf ( stderr, "You must have at least 80 cols.\n" );
		quit ( 0 );
	}

	if ( LINES < 8 ) {
		fprintf ( stderr, "You must have at least 8 lines.\n" );
		quit ( 0 );
	}

	for ( i = 1; i < NSIG; ++i )
		if ( i == SIGCLD )
			continue;
		else
			( void ) signal ( i , quit );

	if ( Num_display_proc > LINES - 6 || Num_display_proc < 0 )
		Num_display_proc = LINES - 6 ;

	curses_refresh();
}

void clear_screen()
{
	erase();
	move ( 0, 0 );
}

int look_lines()
{
	return ( LINES );
}

int look_cols ()
{
	return ( COLS );
}

void update_first_line( nproc, dinfo )
int nproc;
struct pst_dynamic *dinfo;
{
	extern struct tm *localtime();
	struct tm actual;
	time_t mclock = time ( ( time_t *)0 );

	long user_processes = 0L, t_size = 0L, t_rss = 0L;

	clear_screen();

	actual = *localtime ( &mclock );
	count_status ( nproc );

	PF ( "[%2d:%.2d%:%.2d] Load averages: %.2f  %.2f  %.2f.  Pages: %d - %s Idle\n" ,
		actual.tm_hour, actual.tm_min, actual.tm_sec,
		dinfo->psd_avg_1_min, dinfo->psd_avg_5_min,
		dinfo->psd_avg_15_min, pages,
		info_cpu ( dinfo->psd_cpu_time ));

	PF ( "%d Processes: %d Sleeping, %d Running, %d Stopped, %d Zombie, %d Others.\n",
		nproc, status_sleep(), status_run(), status_stop(),
		status_zombie(), status_other() );

	PF ( "Memory: %s  ", get_dimension ( dinfo->psd_rm) );
	PF ( "( %s ) real,  ", get_dimension ( dinfo->psd_arm ) );
	PF ( "%s  ", get_dimension ( dinfo->psd_vm ) );
	PF ( "( %s ) virtual, ", get_dimension ( dinfo->psd_avm ) );
	PF ( "%s free\n", get_dimension ( dinfo->psd_free ) );

	if ( look_uid >= 0 ) {
		get_user_sum( nproc, &user_processes, &t_size, &t_rss);
		curses_in_reverse();
		PF ( "UID %d: %ld processes,", look_uid, user_processes );
		PF ( "  %s  TOTAL SIZE,", get_dimension ( t_size ) );
		PF ( "  %s  TOTAL RSS.", get_dimension ( t_rss ) );
		curses_out_reverse();
	}
		
}

void curses_help()
{
	static int first = 1;
	int page = 0, car = 0;

	if ( LINES < 24 ) {
		curses_error ( "Sorry, you must have at least 24 lines." );
		curses_refresh();
		( void )sleep ( 1 ); /* refresh too fast */
		return;
	}

	if ( !first )
		car = ( int )'f';  /* The first time start page 0 else 1 */
	else
		car = ( int )'B';
	
	first = 0;

	do {
		switch ( car ) {
		case 'f':
		case ' ':
			if ( page < 2 )
				++page;
		break;
		case 'b':
			if ( page > 0 )
				--page;
		break;
		case 'B':
			page = 0;
		break;	
		default:
		break;
		}

		switch ( page ) {
		case 0:
		clear_screen();
		PF ( "YAMM ( Yet Another Micro Monitor ) V2.4 Jun 1992.\n" );
		PF ( "By Marangoni Andrea.\n" );
		PF ( "E-mail: marangon@ghost.dsi.unimi.it.\n");
		PF ( "Department of Computer Science at Milano.\n\n" );
		PF ( "Explanation of fields:\n\n" );
		PF ( "S           - Status of process.\n");
		PF ( "    'S' Sleep, 'R' Running or waiting for Cpu, 'T' Stopped,\n");
		PF ( "    'Z' Zombie, 'I' Being created, 'O' Other ( forking, exiting, etc. ).\n");
		PF ( "\nUSER        - Name of user.\n" );
		PF ( "SUID        - Effective user id ( an '*' means that euid != uid ).\n");
		PF ( "PID         - Process id.\n" );
		PF ( "NI          - Nice.\n" );
		PF ( "PRI         - Priority.\n" );
		PF ( "SIZE        - Size of process ( data + text + stack ).\n");
		PF ( "RSS         - Resident set size ( private pages only ).\n");
		PF ( "STIME       - Start time.\n");
		PF ( "\"u|s| \"TIME - ( 'TIME' Total time, 'uTIME' User time, 'sTIME' System time ).\n" );
		PF ( "CMD         - Command.\n" );
		move ( LINES - 1 , 0 );
		PF ( "Type 'f' to next page or <RETURN> to exit." );
		break;

		case 1:
		clear_screen();
		PF ( "YAMM ( Yet Another Micro Monitor ) V2.4 Jun 1992.\n\n" );
		PF ( "\"f\" Forward  1 page.\n" );
		PF ( "\"b\" Backward 1 page.\n" );
		PF ( "\"B\" Go to page 0.\n\n" );
		PF ( "\"s\" Sort by 's'tart time.\n" );
		PF ( "\"t\" Sort by process 't'ime.\n" );
		PF ( "\"c\" Sort by process %%cpu time, default.\n" ); /* HV */
		PF ( "\"d\" Sort by process size ( 'd'ata + text + stack ).\n" );
		PF ( "\"D\" Sort by process resident set size ( private pages ).\n" );
		PF ( "\"i\" Invert sorting order.\n");
		PF ( "\"<SPACE>\" Disable sorting.\n\n" );
		PF ( "\"m\" Display only 'm'y processes. Press \"m\" again to return to previous state.\n");
		PF ( "\"r\" Allow display of 'r'oot processes ( default = off ).\n" );
		PF ( "\"E\" Show processes with UID != EUID. ( \"E\" again to return to previous state ).\n" );
		PF ( "\"w\" Like \"who\". ( \"w\" again or <RETURN> to return to previous state ).\n" );
		PF ( "\"e\" Display the entries in utmp.\n\n" );
		PF ( "\"P\" Kill process ( After choose it with slide bar ).\n" );
		PF ( "\"K\" Kill user ( After choose it with slide bar ).\n " );
		move ( LINES - 1 , 0 );
		PF ( "Type 'f' to next page, 'b' to previous page or <RETURN> to exit." );
		break;

		case 2:
		clear_screen();
		PF ( "YAMM ( Yet Another Micro Monitor ) V2.4 Jun 1992.\n\n" );
		PF ( "\"u\" Restrict to a specific 'u'id ( You may choose it with slidebar ).\n" );
		PF ( "    Type \"m\"  to return to previous state.\n" );
		PF ( "    Type \"I\" for full user info.\n\n" );
		PF ( "\"S\" Display only System time in field \"TIME\".\n");
		PF ( "\"U\" Display only User time in field \"TIME\".\n\n");
		PF ( "\"%%\" Switch between  STIME and WCPU fields.\n" );
		PF ( "    If you have >= 90 cols both fields are displayed.\n" );
		PF ( "\"N\" Display only \"number\" processes.\n" );
		PF ( "\"p\" Analyze a single process ( Type <SPACE> to return to previous state ).\n" );
		PF ( "\"C\" Display configuration of machine.\n" );
		PF ( "\"V\" Virtual memory info.\n" );
		PF ( "\"W\" Loop forever waiting \"second\" for each loop.\n" );
		PF ( "\"/\" Search \"string\" in command line. NULL string or <SPACE> to annull.\n" );
		PF ( "\"R\" Renice process ( You must be root :-) ).\n" );
		PF ( "\"j\" Move slide bar up.\n" );
		PF ( "\"k\" Move slide bar down.\n\n" );
		PF ( "\"Q\" or \"CTRL-D\" or ESCAPE to exit.\n" );
		move ( LINES - 1 , 0 );
		PF ( "Type 'b' to previous page or <RETURN> to exit." );
		break;
		default:
		break;
	}
	curses_refresh();
	} while ( ( car = getch() ) != 'q' && car != '\n' ); 

}

void curses_refresh()
{
	refresh();
}

void refresh_all ()
{
	clear();
	touchwin( stdscr );
	curses_refresh();
}

int curses_wait( second )
int second;
{
	struct timeval timeout;
	fd_set input_set;

	timeout.tv_sec = ( second <= 0 ) ?  1L : ( long )second;
	timeout.tv_usec = 0L;

	FD_ZERO ( &input_set );
	FD_SET ( fileno ( stdin ), &input_set );

	if ( select ( ( size_t )(fileno ( stdin ) + 1), ( int *)&input_set,
		( int *)0, ( int *)0, &timeout ) == -1 ) {
			curses_error( "Select" );
		return ( 0 );
	}


	if ( FD_ISSET ( 0, &input_set ) ) {
		int i = getch();

		if ( i == 'q' || i == 'Q' || i == CTRL('D') || i == 27 )
			quit ( 0 );

		if ( i == CTRL('L') || i == CTRL('R') ) {
			refresh_all();
			return ( 0 );
		}


		if ( i == '?' || i == 'h' )
			curses_help();

		mpreprocess ( i, NULL, NULL );
		return ( i );
	}
	return ( 0 );
}

#define BUF 128

long ask_curses ( ask )
char *ask;
{
	extern long atol();
	char buf [ BUF ];
	register int i;

	move ( LINES - 1 , 0 );
	clrtoeol();
	PF ( ask );
	curses_refresh();
	getstr ( buf );

	/* skip space and tab */
	for ( i = 0; buf [ i ] == ' ' || buf [ i ] == '\t'; ++i );

	if ( buf[ i ] == '\n' || buf[ i ] == '\0' )
		return ( -1L );

	return ( atol ( buf + i ) );
}

char *curses_get_string ( ask )
char *ask;
{
	static char buf [ BUF ];
	register int i;

	move ( LINES - 1 , 0 );
	clrtoeol();
	PF ( ask );
	curses_refresh();
	getstr ( buf );

	/* skip space and tab */
	for ( i = 0; buf [ i ] == ' ' || buf [ i ] == '\t'; ++i );

	if ( buf[ i ] == '\n' || buf [ i ] == '\0' )
		return ( NULL );

	return ( buf + i  );
}


void curses_error ( message )
char *message;
{
	move ( 3, 0 );
	clrtoeol();
	curses_in_reverse();
	PF ( message );
	curses_out_reverse();
}

/* if reverse don't work comment "standout()" and "standend()" */
void curses_in_reverse()
{
	standout();
}

void curses_out_reverse()
{
	standend();
}


#define HOSTNAMELEN 65 /* 64 + 1 */
void curses_print_help()
{
	static int yet = 0;
	static char host_name [ HOSTNAMELEN ];

	if ( !yet ) {
		host_name [ 0 ] = '\0';
		if ( gethostname ( host_name, ( size_t ) HOSTNAMELEN ) == -1 )
			host_name [ 0 ] = '\0';
		yet = 1;
		host_name [ HOSTNAMELEN - 1 ] = '\0';
	}

	move ( LINES - 1 , 0 );
	clrtoeol();

	if ( host_name [ 0 ] )
		PF ( "[ %s ] Type <?> for help." , host_name );
	else
		PF ( "Type < ? > for help." );
}

#else /* NO_CURSES */

long ask_curses( msg )
char *msg;
{
	return ( -1L ); 
}

char *curses_get_string( msg )
char *msg;
{
	return ( NULL );
}

int look_lines()
{
	return ( 0 ); /* Not necessary only for lint */
}

int look_cols()
{
	if ( long_format )
		return ( 90 ); 
	else
		return ( 80 );
}

#endif

