#define _MAIN
#include <pwd.h>
#include <time.h>
#include <string.h>
#include <sys/param.h>
#include <sys/user.h>
#include <sys/proc.h>
#include <sys/stat.h>
#include <sys/ptyio.h>
#include <sys/sysmacros.h>
#include "define.h"
#include "extern.h"

#ifndef lint
static char *_yamm1 =
	"@(#) YAMM ( Yet Another Micro Monitor ) V2.4 by Marangoni Andrea";
static char *_yamm2 =
	"@(#) E-mail: marangon@ghost.dsi.unimi.it";
#endif


#ifndef NO_CURSES
static char *cmd_line = "mN:CVdEDeiswtcr%u:SUP:W:/:";
#else
static char *cmd_line = "mN:CVdDEeiswtcr%Lu:SUP:W:/:";
#endif

static char *get_command();
static void Look_process();

int main( ac, av )
int ac;
char *av[];
{
	int ret,i,j;
	struct pst_dynamic dinfo;

#ifndef NO_CURSES
	wait_second = 2;
#endif

/* Thanks to Harald Vogt <harald@cs.ruu.nl> for ideas and suggestions */

	order = compare_sched_time; /* HV */
	mpreprocess ( ac, av, cmd_line );

	if ( ( ret = PS_proc ( pbuf, MAX_PROC, 0 )) == -1 )
		( void )perror ( "Pstat: PSTAT_PROC" ), exit ( 1 );

	set_users ();

#ifndef NO_CURSES
	( void )fflush ( stdout );
	setup_tty();
	init_curses();
#else
	count_status ( ret );
	printf ( "Number of processes in system: %d\n",ret );
	printf ( "    %d Running, %d Sleeping, %d Stopped, %d Zombie, %d Other.\n",
		status_run(), status_sleep(),status_stop(),
		status_zombie(),status_other());
#endif

	goto first_time;  /* Not really needed */

	do {

#ifndef NO_CURSES
		curses_print_help();
		curses_refresh();
#else
		fflush ( stdout );
#endif
		
#ifndef NO_CURSES
		( void )curses_wait ( wait_second );
#else
		( void )sleep ( wait_second <= 0 ? 1 : wait_second );
#endif

first_time:   /* First time avoid sleep */

		if ( ( ret = PS_proc ( pbuf, MAX_PROC, 0 ) ) == -1 )
			( void )perror ( "Pstat: PSTAT_PROC" ), exit ( 1 );

		PS_dynam ( &dinfo );

#ifndef NO_CURSES
		update_first_line( ret, &dinfo  );	
#define PF (void)printw
#else
		if ( Num_display_proc == -1 )
			Num_display_proc = ret;
#define PF (void)printf
#endif

		if ( who ) {
			Print_Who( ret ); 
			continue;
		}

		if ( look_utmp ) {
			examine_utmp();
			continue;
		}

		if ( look_process >= 0 ) {
			Look_process ( look_process, ret );
			continue;
		}

		if ( print_configuration ) {
			Print_Conf( ret, &dinfo );
			continue;
		}


		if ( vm_config ) {
			Print_vm();
			continue;
		}
	
		if ( order )
			( void ) qsort ( ( void *)pbuf, ( size_t )ret,
			( size_t )sizeof ( struct pst_status ), order );


		if ( look_cols() >= 90 || wcpu )
			PF ( "\nS  USER      SUID  PID  NI PRI    SIZE     RSS  %%WCPU   %s%cTIME CMD\n",
				look_cols() >= 90 ? "STIME    " : "",
				system_time_include & user_time_include ? ' ':
				user_time_include ? 'u' : 's');
		else
			PF ( "\nS  USER      SUID  PID  NI PRI    SIZE     RSS   STIME    %cTIME CMD\n",
				system_time_include & user_time_include ? ' ':
				user_time_include ? 'u' : 's');


		for ( i = j = displayed = 0, reverse_index = -1; i < ret; ++i) {

			if ( look_uid >= 0L && pbuf [ i ].pst_uid != look_uid )
				continue; /* -U option */

			if ( !allow_root && !pbuf [ i ].pst_uid )
				continue; /* !-R option */

			if ( set_euid  && pbuf [ i ].pst_uid == pbuf [ i ].pst_suid)
				continue; /* -E option */

			if ( sstring && !reg_match ( pbuf [ i ].pst_cmd ))
				continue;

#ifndef NO_CURSES
			if ( j++ < pages*Num_display_proc )
				continue;
#endif
			if ( Num_display_proc >= 0 && displayed >= Num_display_proc )
				break;

			displayed++;

#ifndef NO_CURSES
			if ( reverse == displayed  ) {
				reverse_index = i;
				curses_in_reverse();	
			}
#endif
		
			PF ( "%c %-*s %c%5ld %5ld %2ld %3ld %s ",
				get_status ( i ),
				MAX_LOGIN,
				get_user ( i ),
				pbuf [ i ].pst_uid != pbuf [ i ].pst_suid ? '*':' ',
				pbuf [ i ].pst_suid,
				pbuf [ i ].pst_pid,
				pbuf [ i ].pst_nice,
				pbuf [ i ].pst_pri,
				get_dimension (  pbuf [ i ].pst_dsize +
					pbuf [ i ].pst_tsize  + pbuf [ i ].pst_ssize ) );

			PF ( "%s ",
				get_dimension ( pbuf [ i ].pst_rssize ));

			/* printw seems don't well work */
			if ( look_cols () >= 90 || wcpu ) {  /* xterm ? */
				double pcpu = pbuf [ i ].pst_pctcpu * 100.00;

				/*  Bad trick here */
				PF ( "%2d.%.2d%% ", ( int )pcpu,
					( int )(( pcpu - ( int )pcpu) * 100.00 ));
			}

			if ( !wcpu )
				PF ( "%s ",
				visual_start_time ( i ));

			PF ( "%s %s\n",
				visual_user_time ( i ),
				long_format ? pbuf [ i ].pst_cmd : get_command ( i ));

#ifndef NO_CURSES
			if ( reverse == displayed )
				curses_out_reverse();	
#endif
		}

		if ( reverse_index == -1 )
			reverse = 0;

		if ( displayed == 0 && look_uid >= 0 && !pages ) {
#ifdef NO_CURSES
			PF ( "No such uid: %d\n", look_uid );
#else
			curses_error ( "No such uid." );
#endif
			look_uid = -1;
		}

#undef PF
	} while ( wait_second >= 0 );


#ifndef NO_CURSES
	quit ( 0 );
#endif
	return ( 0 );
}

static char *get_command ( index )
int index;
{
	extern char *strtok();
	char *first_string = pbuf [ index ].pst_cmd ?
		strtok (  pbuf [ index ].pst_cmd , " \t\n" ) : "???";

	char *last_cmd = strrchr ( first_string, '/' );

	return (  last_cmd ? last_cmd + 1 : first_string );
}

static void Look_process ( process, num )
long process;
int num;
{
	extern char *ctime();
	struct pst_status myps;
	register int i;

	for ( i = 0; i < num && process!= pbuf[ i ].pst_pid; ++i );

#define PP myps

#ifdef NO_CURSES
#define PF (void)printf

#else
#define PF (void)printw
	if ( look_lines() < 24 ) {
		curses_error( "Sorry, you must have at least 80 cols." );
		look_process = -1;
		return;
	}


#endif

	if ( PS_proc ( &myps, 0, process ) == -1 )
#ifdef NO_CURSES
		( void )perror ( "Pstat: PSTAT_PROC" ),exit ( 1 );
#else
		{
			curses_error( "No such process......." );
			look_process = -1;
			return;
		}
#endif

	PF ( "\nUID                                  : %ld  %s\n",
		PP.pst_uid, get_user ( i ));
	PF ( "Process SUID                         : %ld\n", PP.pst_suid );
	PF ( "Process ID                           : %ld\n", PP.pst_pid );
	PF ( "Parent process ID                    : %ld\n", PP.pst_ppid );
	PF ( "Process data, text, stack size       : %s, ",
		get_dimension ( PP.pst_dsize ));
	PF ( "%s, ",
		get_dimension ( PP.pst_tsize ));
	PF ( "%s\n",
		get_dimension ( PP.pst_ssize ));
	PF ( "Resident set size for process        : %-5ld Pages ( %s )\n",
		PP.pst_rssize, get_dimension ( PP.pst_rssize ));
	PF ( "Nice, Priority                       : %ld, %ld\n",
		PP.pst_nice,
		PP.pst_pri );
	PF ( "Process group                        : %ld\n", PP.pst_pgrp );
	PF ( "User time spent executing            : %ld Sec.\n", 
		PP.pst_utime );
	PF ( "System time spent executing          : %ld Sec.\n",
		PP.pst_stime );
	PF ( "Start time                           : %s",
		ctime ( &PP.pst_start ));
	PF ( "Current status                       : %c\n",
		get_status ( i ));
	PF ( "Command the proc is running          : %s\n", PP.pst_cmd );
	PF ( "Terminal associated to the process   : %s\n",
		get_tty ( PP.pst_term.psd_major, PP.pst_term.psd_minor ));
	PF ( "Resident time for scheduling         : %ld\n", PP.pst_time );
	PF ( "Ticks of cpu time                    : %ld\n",
		PP.pst_cpticks);
	PF ( "Total ticks for life of process      : %ld\n",
		PP.pst_cptickstotal );
	PF ( "%%cpu for this process during p_time  : %f%%\n",
		PP.pst_pctcpu * 100.00 );

#undef PP
#undef PF
}
